/*
** CENSPAR.G
**  Purpose: Apply nonperiodic linear Census-SA filter to series in VQ format
**  Format: {vysai,censlen}=censpar{vy,henwidth};
**    Input: vy: Txs series of observations;
**           henwidth: width of Hendersonfilter;
**   Output: vysa: (T-k)xs series of SA observations
**           censlen: length of filter-1;
**
**  Author: Marius Ooms
**  Date:   14/10/96
**  email:  ooms@econometriclinks.com
**  URL:    http://www.econ.vu.nl/econometriclinks/ooms/programs/unempcor.zip
**  Reference:
**    Ooms, Marius and Franses, Philip Hans, (1997), "On Periodic Correlations
**    Between Estimated Seasonal and NonSeasonal Components in Germand and
**    US Unemployment", Journal of Business and Economic Statistics,  15, 470-481
*/
proc (2) = censpar(vy,henwidth);
local vysa,y,censf,s,censlen,y0;
 y=vecr(vy);
 s=cols(vy);
 censf=censfilt(henwidth,s);
 censlen=rows(censf)-1;
 y=trimr(conv(y,censf,0,0),censlen,censlen);
 y=trimr(y,(rows(y)%s)/2,(rows(y)%s)/2);
 vysa=reshape(y,rows(y)/s,s);
 retp(vysa,censlen);
endp;
