/*
** PVARSIM.G
**  Purpose: Simulate VARX conditional on starting values
**  Format:  {ysim}=parvarsim(phi,beta,omega,x,y0,repl);
**   Input:  phi: (pg+1)xg: Matrix with ar-coeficients VAR: (0)|(1)|...|(p)
**             beta: gxk matrix with coefficient exogenous variables
**             omega: gxg matrix with variance/covariance matrix error terms
**             x:    kxT vector with values exogenous variables
**             y0:   gxp matrix with starting values endogenous variables
**             ysim: gx(repl(T+p)) matrix with simulated values
**             repl: number of replications
**
**  Author: Marius Ooms
**  Date:   14/10/96
**  email:  ooms@econometriclinks.com
**  URL:    http://www.econ.vu.nl/econometriclinks/ooms/programs/unempcor.zip
**  Reference:
**    Ooms, Marius and Franses, Philip Hans, (1997), "On Periodic Correlations
**    Between Estimated Seasonal and NonSeasonal Components in Germand and
**    US Unemployment", Journal of Business and Economic Statistics,  15, 470-481
**
*/
proc  PVARSIM(phi,beta,omega,x,y0,repl);
local i,j,l,n,T,p,k,phirf,phi0inv,omegarfl,betarf,eps,g,ysim,x0;
g=cols(phi);
p=(rows(phi)./g)-1;
k=rows(x);
T=cols(x);
phi0inv=inv(phi[1:g,1:g]);
omegarfl=(chol(phi0inv*omega*(phi0inv')))';
betarf=phi0inv*beta;
phirf=zeros(p*g,g);                                 @ Reduced form @
i=0;
 do while i<((p-1)*g+0.5);
 phirf[i+1:i+g,.]=phi0inv*phi[(i+g+1):(i+2*g),.];
 i=i+g;
endo;
x0=zeros(k,p)~x;
eps=(omegarfl*rndn(g,repl*(T+p)));
 ysim=zeros(g,repl*(T+p));
j=1;
do while j<repl+0.5;
 i=1+(j-1)*(T+p);
 n=i-1;
 ysim[.,i:i+p-1]=y0;
 i=i+p;
 do while i<(j*(T+p)+0.5);
  l=1;
  do while l<p+0.5;
   ysim[.,i]=ysim[.,i]+phirf[(l-1)*g+1:(l*g),.]*ysim[.,i-l];
   l=l+1;
  endo; @ l @
   ysim[.,i]=ysim[.,i]+betarf*x0[.,i-n]+eps[.,i];
   i=i+1;
 endo; @ i @
  j=j+1;
endo; @ j @
retp(ysim');
endp(ysim);
