/*
**  Purpose of this program is to generate values from structural time series
**    Models
*/
/*
**  Trigonometric seasonality
*/
library pgraph;
/* stmrig:
** Purpose generate random values from structural time series model
**         with trigonometric seasonality
**  Input: seps  : scalar  : standard deviation innovation irregular
**         seta  : scalar  : standard deviation innovation level
**         szeta : scalar  : standard deviation innovation slope
**         somega: scalar  : standard deviation innovation seasonal
**         irr   : scalar  : mean irregular
**         sirr  : scalr   : standard error irregular
**         lvl   : scalar  : mean level
**         slvl  : scalar  : standard error level
**         slp   : scalar  : mean slope
**         sslp  : scalar  : standard error slope
**         sea   : sx1     : means seasonal components season i
**         ssea  : (s-1)x1 : standard error seasonal components ssea ??
**
*/
proc (5)=stmtrig(eps,seps,seta,szeta,somega,irr,sirr,lvl,slvl,slp,
                      sslp,sea,ssea);
local y,mu0,beta0,gamma0,mu,beta,gamma1,gamma2,s,trafmat,n;
 trafmat=0.5~0~-0.5|0~0.5~0|0.25~-0.25~0.25;
 /*
   transforms coefficients dummy variables Q1t,Q2t,Q3t,starting values to
   trigonometric value starting values cos(2t/2),sin(2t/2),(-1)^t,
   not used yet
 */
 s=rows(sea);
 n=rows(eps);
 mu0  = lvl+slvl*rndn(1,1);
 /* mu0   =lvl+(0.25~0.25~0.25)*sea+slvl*rndn(1,1); */
 beta0 =slp+sslp*rndn(1,1);
 gamma0=sea; /* +ssea.*rndn(s,1); */
 beta=recserar(rndn(n,1)*szeta,beta0,1);
 mu  =recserar(beta+rndn(n,1)*seta,mu0,1);
 gamma1 = recserar(rndn(n,2)*(1|1).*somega,gamma0[3|4,1],(0|-1));
 gamma2 = recserar(rndn(n,1)*somega,gamma0[4,1],-1);
 y = mu + gamma1 + gamma2 + eps;
 retp(y,mu,gamma1,gamma2,beta);
endp;
/*
**   Set paramaters
*/
 n=10000;
 eps=zeros(n,1);

/* German model no-logs, dummies trigonometric seasonality */
/*
 seps  = 0;
 seta  =14.584;
 szeta =33.091;
 somega= 4.6253;
 irr  =0      ;sirr=0;
 lvl  = 2263.2; slvl = 49.417;
 slp  = 8.9645; sslp = 51.497;
 sea  = 182.17|-8.1692|84.396;
 sea  = 266.6|-92.56|-97.77|-76.23;
 ssea = 12.649|13.139|9.2358;
 lvl = 2263;
 sea = 182|-8|84;
 eps[20]=181;
 eps[21]=104;
 eps[32]=-23;
 eps[24]=-36;
*/
/* USA model logs, no dummies, trigonometric seasonality */
 seps  = 0;
 seta  =0.028270;
 szeta =0.037569;
 somega= 0.0035395;
 irr  =0      ;sirr=0;
 lvl  = 8.8306; slvl = 0.059862;
 slp  = -0.028459; sslp = 0.059913;
 sea  = 0.037122|0.019483|0.026164;
 sea  = 0.06329|-0.006681|-0.01096|-0.04565;
 ssea = 0.012387|0.012700|0.0088165;
/* USA model logs, dummies  67 75, trigonometric seasonality */
 seps  = 0;
 seta=            0.026874;
 szeta=           0.029559;
 somega=           0.0037411;
 irr  =0      ;sirr=0;
 lvl  = 8.8306; slvl = 0.059862;
 slp  = -0.028459; sslp = 0.059913;
 sea  = 0.037122|0.019483|0.026164;
 sea  = 0.06329|-0.006681|-0.01096|-0.04565;
 ssea = 0.012387|0.012700|0.0088165;
/*
 eps[20]=181;
 eps[21]=104;
 eps[32]=-23;
 eps[24]=-36;
*/
 /*  Put things to zero if you like
    somega=0;
    seta=0;
    szeta=0;
    irr =0;   sirr = 0;
    slvl=0;
    sslp=0;
    slp=0;
    ssea=zeros(3,1);
 */
 {y,mu,gamma1,gamma2,beta}=stmtrig(eps,seps,seta,szeta,somega,irr,sirr,
           lvl,slvl,slp,sslp,sea,ssea);
 dy=trimr(y,1,0)-trimr(y,0,1);
 d4y=trimr(y,4,0)-trimr(y,0,4);
begwind;
 window(4,1,0);
 title("mu(t)");
 xy(seqa(1,1,n),mu);
 nextwind;
 title("gamma1(t)");
 xy(seqa(1,1,n),gamma1);
 nextwind;
 title("gamma2(t)");
 xy(seqa(1,1,n),gamma2);
 nextwind;
 title("beta(t)");
 xy(seqa(1,1,n),beta);
endwind;
begwind;
 window(3,1,0);
 title("y(t)");
 xy(seqa(1,1,n),y);
 nextwind;
 title("y(t)-y(t-1)");
 xy(seqa(1,1,n-1),dy);
 nextwind;
 title("y(t)-y(t-4)");
 xy(seqa(1,1,n-4),d4y);
endwind;
 format(16,10);
 output file = "yud.asc" reset;
 y;
 output off;
end;
